#!/usr/bin/perl -w
# DESCRIPTION: Perl ExtUtils: Test called via 'project_bin'
#
# Copyright 2006-2010 by Wilson Snyder.  This program is free software;
# you can redistribute it and/or modify it under the terms of either the GNU
# Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.

print "hello world!\n";
